use crate::helpers::db;
use crate::models::{artists::Artists, spotify};
use crate::schema::{playlists_tracks, tracks};
use chrono::NaiveDateTime;
use diesel::prelude::*;
use diesel::result::Error;
use serde::{Deserialize, Serialize};

#[derive(
    AsChangeset, Debug, Deserialize, Serialize, Queryable, Identifiable, Selectable, Clone,
)]
#[diesel(table_name = tracks)]
pub struct Tracks {
    pub id: String,
    pub title: String,
    pub duration_ms: i32,

    pub created_at: Option<NaiveDateTime>,
    pub updated_at: Option<NaiveDateTime>,

    pub spotify_id: Option<String>,
    pub tidal_id: Option<String>,
}

#[derive(AsChangeset, Clone, Debug, Deserialize, Insertable)]
#[diesel(table_name = tracks)]
pub struct NewTrack {
    pub id: String,
    pub title: String,
    pub duration_ms: i32,

    pub spotify_id: Option<String>,
    pub tidal_id: Option<String>,
}

impl Tracks {
    pub fn find(id: String) -> Result<Self, Error> {
        let conn = &mut db::connection()?;
        let playlist = tracks::table.filter(tracks::id.eq(id)).first(conn)?;
        Ok(playlist)
    }

    pub fn create(track: NewTrack) -> Result<Self, Error> {
        let conn = &mut db::connection()?;
        let playlist = diesel::insert_into(tracks::table)
            .values(track)
            .get_result(conn)?;
        Ok(playlist)
    }

    pub fn create_or_update(track: NewTrack) -> Result<Self, Error> {
        let conn = &mut db::connection()?;
        let playlist = diesel::insert_into(tracks::table)
            .values(track.clone())
            .on_conflict(tracks::id)
            .do_update()
            .set(track)
            .get_result(conn)?;
        Ok(playlist)
    }

    pub fn get_artists(&self) -> Result<Vec<Artists>, Error> {
        let artists = Artists::find_by_track(&self.id)?;
        Ok(artists)
    }

    pub fn with_artists(track: Tracks) -> Result<TracksWithArtists, Error> {
        let artists = track.get_artists()?;

        Ok(TracksWithArtists {
            id: track.id,
            title: track.title,
            duration_ms: track.duration_ms,

            artists,

            spotify_id: track.spotify_id,
            tidal_id: track.tidal_id,
        })
    }
}

impl From<spotify::Track> for Tracks {
    fn from(value: spotify::Track) -> Tracks {
        Tracks {
            id: value.external_ids.isrc,
            title: value.name,
            duration_ms: value.duration_ms,
            spotify_id: Some(value.id),
            tidal_id: None,
            created_at: None,
            updated_at: None,
        }
    }
}

#[derive(Deserialize, Serialize)]
pub struct TracksWithArtists {
    pub id: String,
    pub title: String,
    pub duration_ms: i32,

    pub artists: Vec<Artists>,

    pub spotify_id: Option<String>,
    pub tidal_id: Option<String>,
}
