use crate::middlewares::error::ErrorResponse;
use crate::middlewares::user::get_user;
use crate::models::playlists::{NewPlaylist, PlaylistCreator, Playlists};
use crate::routes::playlists::GetPlaylistResponse;
use actix_web::{get, post, web, HttpRequest, HttpResponse, Scope};
use serde::Serialize;

pub fn routes() -> Scope {
    web::scope("/me")
        .service(me)
        .service(me_playlists)
        .service(create_playlist)
}

#[derive(Serialize)]
struct Response {
    id: String,
    name: String,
    email: String,
}

#[get("")]
async fn me(req: HttpRequest) -> Result<HttpResponse, ErrorResponse> {
    let user = get_user(req)?;

    Ok(HttpResponse::Ok().json(Response {
        id: user.id,
        name: user.name,
        email: user.email,
    }))
}

#[get("/playlists")]
async fn me_playlists(req: HttpRequest) -> Result<HttpResponse, ErrorResponse> {
    let user = get_user(req)?;

    let playlists = Playlists::find_for_user(&user.id, false)?;
    let playlists = playlists
        .into_iter()
        .map(|playlist| {
            let creator = playlist.get_creator().unwrap();

            let tracks = playlist.get_tracks().unwrap();
            let (duration, tracks_count) = playlist.get_data(&tracks);

            GetPlaylistResponse {
                id: playlist.id.to_string(),
                name: playlist.name.to_string(),
                playlist_type: playlist.playlist_type.to_string(),
                parent_id: playlist.parent_id,
                creator: PlaylistCreator {
                    id: creator.id,
                    name: creator.name,
                },
                tracks,
                tracks_count,
                duration,
            }
        })
        .collect::<Vec<GetPlaylistResponse>>();

    Ok(HttpResponse::Ok().json(playlists))
}

#[post("/playlists")]
async fn create_playlist(
    req: HttpRequest,
    mut playlist: web::Json<NewPlaylist>,
) -> Result<HttpResponse, ErrorResponse> {
    let user = get_user(req)?;

    playlist.creator_id = user.id;

    Playlists::create(playlist.into_inner())?;

    Ok(HttpResponse::Accepted().finish())
}
