use crate::middlewares::error::ErrorResponse;
use crate::middlewares::user::get_user_option;
use crate::models::playlists::{PlaylistCreator, PlaylistTrack, Playlists};
use actix_web::{get, web, HttpRequest, HttpResponse, Scope};
use reqwest::StatusCode;
use serde::{Deserialize, Serialize};

pub fn routes() -> Scope {
    web::scope("/playlists").service(get_playlist)
}

#[derive(Deserialize, Serialize)]
pub struct GetPlaylistResponse {
    pub id: String,
    pub name: String,
    pub playlist_type: String,
    pub parent_id: Option<String>,
    pub creator: PlaylistCreator,
    pub tracks: Vec<PlaylistTrack>,
    pub tracks_count: usize,
    pub duration: usize,
}

#[get("/{playlist_id}")]
async fn get_playlist(
    req: HttpRequest,
    path: web::Path<String>,
) -> Result<HttpResponse, ErrorResponse> {
    let playlist_id = path.into_inner();
    let playlist = Playlists::find(playlist_id.as_str())?;

    if !playlist.can_see(get_user_option(req)) {
        return Err(ErrorResponse::new(
            "Playlist not found",
            StatusCode::NOT_FOUND,
        ));
    }

    let creator = playlist.get_creator()?;

    let tracks = playlist.get_tracks()?;
    let (duration, tracks_count) = playlist.get_data(&tracks);

    Ok(HttpResponse::Ok().json(GetPlaylistResponse {
        id: playlist.id.to_string(),
        name: playlist.name.to_string(),
        playlist_type: playlist.playlist_type.to_string(),
        parent_id: playlist.parent_id,
        creator: PlaylistCreator {
            id: creator.id,
            name: creator.name,
        },
        tracks,
        tracks_count,
        duration,
    }))
}
